@@ -5,8 +5,6 @@ if sys.version_info[0] < 3:
 else:
     from urllib.parse import urlparse
 
-import awkward0 as awkward
-import uproot3 as uproot
 
 input_filenames_argument_name = 'input_filenames'
 tree_name_argument_name = 'tree_name'
@@ -111,14 +109,8 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
                     + '}')
         return node
 
-    def get_globals(self):
-        return globals()
-
     def resolve_id(self, id):
-        if (id in ('True', 'False', 'None')
-           or id in self._id_scopes
-           or id in self.get_globals()
-           or id in ('abs', 'all', 'any', 'len', 'max', 'min', 'sum')):
+        if id in __builtins__ or id in self._id_scopes:
             return id
         else:
             raise NameError('Unknown id: ' + id)
@@ -205,20 +197,41 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         if hasattr(node, 'short_circuit') and node.short_circuit is True:
             node.rep = value_rep + '[' + slice_rep + ']'
         else:
-            node.rep = ('(' + value_rep + '[' + value_rep + '.columns[' + slice_rep + ']]'
-                        + ' if isinstance(' + value_rep + ', awkward.Table)'
-                        + ' and ' + value_rep + '.istuple else ' + value_rep
-                        + '[' + slice_rep + '])')
+            if isinstance(node.slice, ast.Slice):
+                node.rep = ('(' + value_rep + '[' + value_rep + '.fields[' + slice_rep + ']]'
+                            + ' if isinstance(' + value_rep + ', ak.Array)'
+                            + ' else ' + value_rep + '[' + slice_rep + '])')
+            elif (isinstance(node.slice, ast.Tuple)
+                  or ((sys.version_info[0] < 3
+                       or (sys.version_info[0] == 3 and sys.version_info[1] < 9))
+                      and isinstance(node.slice, ast.ExtSlice))):
+                raise NotImplementedError('Multidimensional slices are not supported')
+            else:
+                if ((sys.version_info[0] < 3
+                     or (sys.version_info[0] == 3
+                         and sys.version_info[1] < 9)) and isinstance(node.slice, ast.Index)):
+                    slice_value = node.slice.value
+                else:
+                    slice_value = node.slice
+                try:
+                    slice_eval = ast.literal_eval(slice_value)
+                    if isinstance(slice_eval, int):
+                        node.rep = ('(' + value_rep + '[' + value_rep + '.fields[' + slice_rep
+                                    + ']]' + ' if isinstance(' + value_rep + ', ak.Array)'
+                                    + ' else ' + value_rep + '[' + slice_rep + '])')
+                    else:
+                        node.rep = value_rep + '[' + slice_rep + ']'
+                except ValueError:
+                    node.rep = ('(' + value_rep + '[' + value_rep + '.fields[' + slice_rep + ']]'
+                                + ' if isinstance(' + value_rep + ', ak.Array)'
+                                + ' and (isinstance(' + slice_rep + ', int)'
+                                + ' or isinstance(' + slice_rep + ', slice))'
+                                + ' else ' + value_rep + '[' + slice_rep + '])')
         return node
 
     def visit_Attribute(self, node):
         value_rep = self.get_rep(node.value)
-        if hasattr(node, 'short_circuit') and node.short_circuit is True:
-            node.rep = value_rep + '.' + node.attr
-        else:
-            node.rep = ('(' + value_rep + '.' + node.attr
-                        + ' if hasattr(' + value_rep + ", '" + node.attr
-                        + "') else " + value_rep + "['" + node.attr + "'])")
+        node.rep = value_rep + '[' + repr(node.attr) + ']'
         return node
 
     def visit_Lambda(self, node):
@@ -268,15 +281,19 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
             else:
                 local_tree_name_rep = ('(lambda key_array: '
                                        + "key_array[key_array[:, 1] == 'TTree'][:, 0])("
-                                       + 'awkward.Table(uproot.open(' + source_rep + '[0])'
-                                       + '.classnames()).unzip()[0])[0]')
-            tree_name_rep = (tree_name_argument_name
-                             + ' if ' + tree_name_argument_name + ' is not None'
-                             + ' else ' + local_tree_name_rep)
+                                       + 'np.atleast_2d((lambda classnames:'
+                                       + ' np.hstack([list(classnames.keys()),'
+                                       + ' list(classnames.values())]))'
+                                       + '(uproot.open(' + source_rep + '[0]).classnames())'
+                                       + '))[0]')
+            tree_name_rep = (tree_name_argument_name + ' '
+                             + 'if ' + tree_name_argument_name + ' is not None '
+                             + 'else ' + local_tree_name_rep)
             node.rep = ('(lambda input_files, tree_name_to_use: '
-                        + "logging.getLogger(__name__).info('Using treename='"
-                        + ' + repr(tree_name_to_use))'
-                        + ' or uproot.lazyarrays(input_files, tree_name_to_use))'
+                        + "(logging.getLogger(__name__).info('Using treename='"
+                        + ' + repr(tree_name_to_use)),'
+                        + ' uproot.lazy({input_file: tree_name_to_use'
+                        + ' for input_file in input_files}))[1])'
                         + '(' + source_rep + ', ' + tree_name_rep + ')')
         else:
             func_rep = self.get_rep(node.func)
@@ -291,16 +308,6 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         if len(node.selector.args.args) != 1:
             raise TypeError('Lambda function in Select() must have exactly one argument, found '
                             + len(node.selector.args.args))
-        if type(node.selector.body) in (ast.List, ast.Tuple):
-            node.selector.body = ast.Call(func=ast.Attribute(value=ast.Name(id='awkward'),
-                                                             attr='Table',
-                                                             short_circuit=True),
-                                          args=node.selector.body.elts)
-        elif type(node.selector.body) is ast.Dict:
-            node.selector.body = ast.Call(func=ast.Attribute(value=ast.Name(id='awkward'),
-                                                             attr='Table',
-                                                             short_circuit=True),
-                                          args=[node.selector.body])
         call_node = ast.Call(func=node.selector, args=[node.source])
         node.rep = self.get_rep(call_node)
         return node
@@ -312,23 +319,8 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         if len(node.selector.args.args) != 1:
             raise TypeError('Lambda function in SelectMany() must have exactly one argument, '
                             'found ' + len(node.selector.args.args))
-        if type(node.selector.body) in (ast.List, ast.Tuple):
-            node.selector.body.elts = [ast.Call(func=ast.Attribute(value=element,
-                                                                   attr='flatten',
-                                                                   short_circuit=True),
-                                                args=[]) for element in node.selector.body.elts]
-        elif type(node.selector.body) is ast.Dict:
-            node.selector.body.values = [ast.Call(func=ast.Attribute(value=dict_value,
-                                                                     attr='flatten',
-                                                                     short_circuit=True),
-                                                  args=[])
-                                         for dict_value in node.selector.body.values]
-        else:
-            node.selector.body = ast.Call(func=ast.Attribute(value=node.selector.body,
-                                                             attr='flatten',
-                                                             short_circuit=True),
-                                          args=[])
         self.visit_Select(node)
+        node.rep = 'ak.flatten(' + node.rep + ')'
         return node
 
     def visit_Where(self, node):
@@ -352,3 +344,7 @@ class PythonSourceGeneratorTransformer(ast.NodeTransformer):
         call_node = ast.Call(func=node.predicate, args=[node.source])
         node.rep = self.get_rep(call_node)
         return node
+
+    def visit_Zip(self, node):
+        node.rep = 'ak.zip(' + self.get_rep(node.source) + ')'
+        return node
